import 'package:flutter/material.dart';

class BuzzSizeUtils {
  // Screen Breakpoints
  static const int xsBreakPoint = 359;
  static const int smBreakPoint = 599;
  static const int mdBreakPoint = 999;
  static const int lgBreakPoint = 1919;
  static const int extraLarge = 1919;

  static T resize<T>({
    required final double maxWidth,
    required final ScreenSizes size,
  }) {
    if (maxWidth > lgBreakPoint && size.xl != null) {
      return size.xl;
    }

    if (maxWidth > mdBreakPoint && size.lg != null) {
      return size.lg;
    }

    if (maxWidth > smBreakPoint && size.md != null) {
      return size.md;
    }

    if (maxWidth <= xsBreakPoint && size.xs != null) {
      return size.xs;
    }

    return size.sm;
  }

  static _Screen of(BuildContext context) {
    return _Screen(size: MediaQuery.of(context).size);
  }
}

class ScreenSizes<T> {
  final T? xs;
  final T sm;
  final T? md;
  final T? lg;
  final T? xl;

  const ScreenSizes({
    this.xs,
    required this.sm,
    this.md,
    this.lg,
    this.xl,
  });
}

class _Screen {
  final Size size;

  const _Screen({required this.size});

  double height({
    final double dividedBy = 1,
    final double margin = 0,
  }) =>
      (size.height / dividedBy) - margin;

  double width({
    final double dividedBy = 1,
    final double margin = 0,
  }) =>
      (size.width / dividedBy) - margin;
}
